/**
 * @PropertiesManager.java 24.11.2010 build: 24.11.2010 Copyright 2010
 *                         yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 *         This file is a part of Sudoku Sudoku is free software; you can
 *         redistribute it and/or modify it under the terms of the GNU General
 *         Public License as published by the Free Software Foundation; either
 *         version 2 of the License, or (at your option) any later version.
 * 
 *         Sudoku is distributed in the hope that it will be useful, but WITHOUT
 *         ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *         FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *         for more details.
 * 
 *         You should have received a copy of the GNU General Public License
 *         along with Sudoku; if not, write to the Free Software Foundation,
 *         Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
package de.yourwebs.sudoku;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Properties;

/**
 * Managed die Optionen-Verwaltung
 */
public class PropertiesManager {

    private String configFilePath;

    private final String header = "# Yourwebs.de - Sudoku\n"
            + "Sudoku Configuration File";

    private Properties defaultProperties = new Properties();
    private Properties properties = new Properties();

    public PropertiesManager() {

        configFilePath = Config.CONFIG_DIR.value
                + System.getProperty("file.separator") + "config";

        defaultProperties.put("sizeSlider", SudokuConstants.GUI_SIZE);
        defaultProperties.put("fontWeight", SudokuConstants.FONT_WEIGHT);
        defaultProperties.put("contrast", SudokuConstants.ACCESSIBILITY);
        defaultProperties.put("mistakeHint", SudokuConstants.MISTAKE_HINT);
        defaultProperties.put("isSuccessSoundPlayable", SudokuConstants.PLAY_SUCCESS_SOUND);
        defaultProperties.put("isSpeechPlayable", SudokuConstants.PLAY_VOICE_OUTPUT);
        defaultProperties.put("difficulty", SudokuConstants.DIFFICULTY);
        defaultProperties.put("mistakeDelay", SudokuConstants.MISTAKE_DELAY);
    }

    /**
     * Legt die Konfigurationsdatei im Verzeichnis .sudoku des
     * home-Verzeichnisses des Users an
     * 
     * @return bei Erfolg true, sonst false
     */
    private boolean makeConfigFile() {

        File file = new File(configFilePath);
        File dir = file.getParentFile();

        if (!dir.isDirectory() && !dir.mkdirs()) {
            System.err.println(dir
                    + " ist kein Verzeichnis und kann nicht erstellt werden!");
            return false;
        }

        try {
            if (file.isFile() && file.canRead() && file.canWrite()) {
                System.out.println(
                        file + " ist vorhanden und lesbar und beschreibbar.");
                return true;
            }
            if (file.createNewFile()) {
                System.out.println(file + " erstellt.");
                return true;
            }
        } catch (IOException e) {
            System.err.println(
                    "Konfigurationsdatei konnte nicht angelegt werden.");
            e.printStackTrace();
        }
        return false;
    }

    /**
     * Speichert die Properties. Wird aufgerufen beim Speichern einzelner
     * Eigenschafts-Werte
     */
    public void saveProperties() {
        saveProperties(null);
    }

    /**
     * Speichert die Properties prop als XML-Datei im Home-Verzeichnis des Users
     * 
     * @param prop
     *            das zu speichernde Properties-Objekt
     */
    public void saveProperties(Properties prop) {
        prop = prop == null ? properties : prop;

        if (configFilePath == null || configFilePath.length() == 0)
            return;

        FileOutputStream propOut = null;
        try {
            propOut = new FileOutputStream(configFilePath);
            properties.putAll(prop);
            prop.storeToXML(propOut, header);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (propOut != null)
                    propOut.close();
            } catch (IOException e) {
            }
        }
    }

    /**
     * Liest die Properties aus der zugeh&ouml;rigen Datei. Wenn diese nicht
     * lesbar ist oder nicht existiert, wird versucht, diese anzulegen und mit
     * den Default-Werten zu belegen.
     */
    public void loadProperties() {

        if (configFilePath == null || configFilePath.length() == 0)
            return;

        File f = new File(configFilePath);
        Enumeration<?> propEnum = defaultProperties.propertyNames();
        String prop;

        if (f.canRead() && f.isFile() && f.length() > 0) {
            System.out.println(f + " ist lesbar");
            FileInputStream propIn = null;
            try {
                propIn = new FileInputStream(configFilePath);
                properties.loadFromXML(propIn);
                // durchlaeuft die Keys der default Properties und prueft, ob
                // diese in
                // den ausgelesenen Properties belegt sind. Falls nicht wird ein
                // Leerstring gespeichert.
                while (propEnum.hasMoreElements()) {
                    prop = (String) propEnum.nextElement();
                    if (properties.getProperty(prop) == null) {
                        properties.setProperty(prop, "");
                    }
                }
                return;
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                try {
                    if (propIn != null)
                        propIn.close();
                } catch (IOException e) {
                }
            }
        }
        // wenn Config-Datei nicht lesbar oder leer defaultProperties laden
        properties = defaultProperties;
        System.out.println(f
                + " ist nicht lesbar oder nicht vorhaden, versuche sie zu erstellen...");
        if (makeConfigFile())
            System.out.println(f + " erfolgreich erstellt");
        saveProperties(properties);
    }

    /**
     * Setzt den Wert value zur Eigenschaft key der Properties. Wenn keine
     * Properties gesetzt sind, werden die Default-Properties geladen
     * 
     * @param key
     *            <code>String</code>, der Properties-Key
     * @param value
     *            <code>String</code>, der Properties-Wert
     */
    public void setProperty(String key, String value) {
        if (properties == null)
            properties = defaultProperties;
        else
            properties.setProperty(key, value);
    }

    /**
     * Liefert den Wert der zu key geh&ouml;renden Eigenschaft
     * 
     * @param key
     *            Schl&uuml;ssel der gefragten Eigenschaft
     * @return String, der Wert der gefragten Eigenschaft
     */
    public String getProperty(String key) {
        if (properties == null)
            properties = defaultProperties;
        return properties.getProperty(key);
    }
}
