package de.yourwebs.sudoku;

import java.util.Random;

import javax.swing.JFrame;

/**
 * @SudokuEngine.java Copyright J&ouml;rg Czeschla, javabeginners.de, 2010
 *                    -2017. All rights reserved
 * @author This file is a part of Sudoku. Sudoku is free software; you can
 *         redistribute it and/or modify it under the terms of the GNU General
 *         Public License as published by the Free Software Foundation; either
 *         version 2 of the License, or (at your option) any later version.
 * 
 *         Sudoku is distributed in the hope that it will be useful, but WITHOUT
 *         ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *         FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *         for more details.
 * 
 *         You should have received a copy of the GNU General Public License
 *         along with Sudoku; if not, write to the Free Software Foundation,
 *         Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

/**
 * Business logic of the game
 * 
 */
public class SudokuEngine extends JFrame {

    /**
     * Number of elements in one row or column of a square
     */
    private int squareRowElements;

    /**
     * Number of elements in one row or column of the entire game
     */
    private int gameRowElements;

    /**
     * Array of the numbers at startup (working array)
     */
    private byte[][] startArray;

    /**
     * Completely and correctly filled array
     */
    private byte[][] resultArray;

    /**
     * Initialises the size of {@link #squareRowElements} and
     * {@link #gameRowElements} with the number of elements per row/col/square.
     * 
     * @param size
     *            <code>int</code> number of elements in a row/col/square.
     */
    protected void initEngine(int size) {
        this.squareRowElements = size;
        gameRowElements = size * size;
        initFields();
    }

    /**
     * Checks if value exists already in a row, column or square. The method's
     * return value makes it possible to display which part of the game (row,
     * col or square) shows <code>value</code> multiple times. Used to show
     * mistakes.
     * 
     * @param row
     *            <code>int</code>, the row
     * @param col
     *            <code>int</code>, the column
     * @param value
     *            <code>int</code>, the value to be checked
     * @return
     *         <p>
     *         <code>int</code> according to the table below
     *         </p>
     *         <table summary="">
     *         <tr>
     *         <td>-1</td>
     *         <td>without match</td>
     *         </tr>
     *         <tr>
     *         <td>0</td>
     *         <td>horizontal match</td>
     *         </tr>
     *         <tr>
     *         <td>1</td>
     *         <td>vertical match</td>
     *         </tr>
     *         <tr>
     *         <td>2</td>
     *         <td>horizontal and vertical match</td>
     *         </tr>
     *         <tr>
     *         <td>3</td>
     *         <td>match in a square</td>
     *         </tr>
     *         <tr>
     *         <td>4</td>
     *         <td>horizontal match and match in a square</td>
     *         </tr>
     *         <tr>
     *         <td>5</td>
     *         <td>vertical match and match in a square</td>
     *         </tr>
     *         <tr>
     *         <td>6</td>
     *         <td>horizontal and vertical match and match in a square</td>
     *         </tr>
     *         </table>
     */
    protected int check(int row, int col, int value) {
        int result = -1;

        if (checkHorizontal(row, col, value)) {
            result += 1;
        }

        if (checkVertical(row, col, value)) {
            result += 2;
        }

        if (checkSquare(row, col, value)) {
            result += 4;
        }
        return result;
    }

    /**
     * Checks if <code>value</code> exists in <code>row</code>.
     * 
     * @param row
     *            <code>int</code>
     * 
     * @param col
     *            <code>int</code>
     * 
     * @param value
     *            <code>int</code> the value to be checked
     * 
     * @return <code>true</code> if <code>value</code> exists in
     *         <code>row</code> otherwise <code>false</code>.
     */
    private boolean checkHorizontal(int row, int col, int value) {
        boolean mistake = false;
        for (int a = 0; a < gameRowElements; a++) {
            if (startArray[row][a] == value && a != col) {
                mistake = true;
                break;
            }
        }
        return mistake;
    }

    /**
     * Checks if <code>value</code> exists in column
     * 
     * @param row
     *            <code>int</code>
     * 
     * @param col
     *            <code>int</code>
     * 
     * @param value
     *            <code>int</code> the value to be checked
     * 
     * @return <code>true</code> if <code>value</code> exists in
     *         <code>col</code> otherwise <code>false</code>.
     */
    private boolean checkVertical(int row, int col, int value) {
        boolean mistake = false;
        for (int a = 0; a < gameRowElements; a++) {
            if (startArray[a][col] == value && a != row) {
                mistake = true;
                break;
            }
        }
        return mistake;
    }

    /**
     * Checks if <code>value</code> exists a square.
     * 
     * @param row
     *            <code>int</code>
     * 
     * @param col
     *            <code>int</code>
     * 
     * @param value
     *            <code>int</code> the value to be checked
     * 
     * @return <code>true</code> if <code>value</code> exists in the square
     *         otherwise <code>false</code>.
     */
    private boolean checkSquare(int row, int col, int value) {
        // oberes, linkes Eck der Box herausfinden
        int yStart = (int) (row / squareRowElements) * squareRowElements;
        int xStart = (int) (col / squareRowElements) * squareRowElements;
        boolean mistake = false;
        for (int a = yStart; a < yStart + squareRowElements; a++)
            for (int b = xStart; b < xStart + squareRowElements; b++) {
                if (startArray[a][b] == value && (a != row && b != col)) {
                    mistake = true;
                    break;
                }
            }
        return mistake;
    }

    /**
     * Computes the solution of the game recursively. An array with length
     * {@link #gameRowElements} is filled with distinct random numbers. While
     * checking for its existance in a row, column or square each time, two
     * identical int arrays are initialized recursively by random numbers.
     * {@link #resultArray} represents the game's solution, {@link #startArray}
     * will be prepared with -1 at some random positions, according to the
     * difficulty of the game. These values are translated to blank spaces
     * displayed on the gui's fields. {@link #startArray} will initialize the
     * game on startup.
     * 
     * @param row
     *            <code>int</code> index of the game's row
     * 
     * @param col
     *            <code>int</code> index of the game's column
     * 
     * @return <code>boolean</code> <code>true</code> if the computing succeeds.
     */
    boolean createSudoku(int row, int col) {
        int k, counter = 0;
        byte[] arr = new byte[gameRowElements];

        while (counter < gameRowElements) {
            arr[counter] = (byte) (new Random().nextInt(gameRowElements) + 1);
            if (!checkRandomEntry(arr))
                counter++;
        }
        for (k = 0; k < gameRowElements; k++) {
            startArray[row][col] = arr[k];
            resultArray[row][col] = arr[k];
            if (!checkForDoubleEntries(row, col)) {
                if (col < gameRowElements - 1) {
                    if (createSudoku(row, col + 1)) {
                        return true;
                    }
                } else {
                    if (row < gameRowElements - 1) {
                        if (createSudoku(row + 1, 0)) {
                            return true;
                        }
                    } else {
                        return true;
                    }
                }
            }
        }
        if (startArray != null)
            startArray[row][col] = 0;
        return false;
    }

    /**
     * Checks if <code>arr</code> only contains distinct values.
     * 
     * @param arr
     *            the array that has to be checked.
     * @return <code>false</code> if there are only distinct values in
     *         <code>arr</code>, otherwise <code>true</code>.
     */
    private boolean checkRandomEntry(byte[] arr) {
        int k = 1, counter = 0;
        while (k <= gameRowElements) {
            for (int i = 0; i < arr.length; i++) {
                if (k == arr[i])
                    counter++;
            }
            if (counter > 1)
                return true;
            k++;
            counter = 0;
        }
        return false;
    }

    /**
     * Checks if there is more than one entry in a row, column or square.
     * 
     * @param row
     *            <code>int</code> the row
     * @param col
     *            <code>int</code> the column
     * @return <code>false</code> if each entry in a row, column or square only
     *         exists one time, otherwise <code>true</code>.
     */
    private boolean checkForDoubleEntries(int row, int col) {
        // check rows and cols
        for (int i = 0; i < startArray.length; i++) {
            int hcount = 0, vcount = 0;
            for (int j = 0; j < startArray[i].length; j++) {
                if (startArray[row][col] == startArray[i][j]
                        && startArray[i][j] != 0) {
                    hcount++;
                }
                if (startArray[row][col] == startArray[j][i]
                        && startArray[j][i] != 0) {
                    vcount++;
                }
            }
            if (hcount > 1 || vcount > 1)
                return true;
        }

        // check squares
        int qcount = 0;
        for (int startX = 0; startX < gameRowElements; startX += squareRowElements) {
            for (int startY = 0; startY < gameRowElements; startY += squareRowElements) {
                qcount = 0;
                for (int i = startX; i < startX + squareRowElements; i++) {
                    for (int j = startY; j < startY + squareRowElements; j++) {
                        if (startArray[row][col] == startArray[j][i]
                                && startArray[j][i] != 0) {
                            qcount++;
                        }
                    }
                }
                if (qcount > 1)
                    return true;
            }
        }
        return false;
    }

    /**
     * Sets <code>anz</code> positions of the start array to -1. Their indices
     * are calculated randomly. Afterwords the according gui-positions are
     * filled by blank spaces.
     * 
     * @param anz
     *            <code>int</code>
     */
    void preallocate(int anz) {
        if (gameRowElements < 3 || startArray == null)
            return;
        int x, y;
        Random random = new Random();
        while (true) {
            x = random.nextInt(gameRowElements);
            y = random.nextInt(gameRowElements);
            if (startArray[x][y] == (byte) -1)
                continue;
            startArray[x][y] = (byte) -1;
            if (isFilled(anz))
                break;
        }
    }

    /**
     * Checks if <code>anz</code> positions of the start array are set to -1.
     * pr&uuml;ft, ob <code>anz</code> Felder des Sudokus auf -1 gesetzt sind
     * 
     * @param anz
     *            <code>int</code> the number of array positions
     * @return <code>true</code> if <code>anz</code> positions are set to -1.
     */
    private boolean isFilled(int anz) {
        int counter = 0;
        for (int i = 0; i < gameRowElements * gameRowElements; i++) {
            if (startArray[i / gameRowElements][i % gameRowElements] == -1)
                counter++;
        }
        if (counter == anz)
            return true;
        return false;
    }

    /**
     * Initialises the array, that will fill the game at startup containing
     * several -1 to display empty fields and the array used to store the
     * solution of the game.
     * 
     * See also {@link #initEngine(int)}, {@link SudokuGUI#initNewSudoku()}
     */
    public void initFields() {
        startArray = new byte[gameRowElements][gameRowElements];
        resultArray = new byte[gameRowElements][gameRowElements];
    }

    public byte[][] getStartArray() {
        return startArray;
    }

    public void setStartArray(byte[][] arr) {
        startArray = arr;
    }

    public byte[][] getResultArray() {
        return resultArray;
    }

    /**
     * Inserts <code>value</code> to <code>{@link #startArray}</code> at
     * position <code>row</code>/<code>col</code>.
     * 
     * @param row
     *            <code>int</code>
     * @param col
     *            <code>int</code>
     * @param value
     *            <code>byte</code> value to be set
     */
    public void setValueToStartArrayPos(int row, int col, byte value) {
        startArray[row][col] = value;
    }

    /**
     * Inserts <code>value</code> to <code>{@link #resultArray}</code> at
     * position <code>row</code>/<code>col</code>.
     * 
     * @param row
     *            <code>int</code>
     * @param col
     *            <code>int</code>
     * @param value
     *            <code>byte</code> value to be set
     */
    public void setValueToResultArrayPos(int row, int col, byte value) {
        resultArray[row][col] = value;
    }

    // public void debug() {
    // System.out.println("StartArray");
    // for (int i = 0; i < gameRowElements; i++) {
    // for (int j = 0; j < gameRowElements; j++) {
    // System.out.print(i + "/" + j + ": " + startArray[i][j] + " ");
    // }
    // System.out.println("\n");
    // }
    // System.out.println("ResultArray");
    // for (int i = 0; i < gameRowElements; i++) {
    // for (int j = 0; j < gameRowElements; j++) {
    // System.out.print(i + "/" + j + ": " + resultArray[i][j] + " ");
    // }
    // System.out.println("\n");
    // }
    // }
}
