/**
 * @SudokuGUI.java 14.10.2010 Copyright 2010 - 2013 yourwebs.de. All rights
 *                 reserved
 * @author J&ouml;rg Czeschla
 *
 *         This file is a part of Sudoku. Sudoku is free software; you can
 *         redistribute it and/or modify it under the terms of the GNU General
 *         Public License as published by the Free Software Foundation; either
 *         version 2 of the License, or (at your option) any later version.
 * 
 *         Sudoku is distributed in the hope that it will be useful, but WITHOUT
 *         ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *         FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *         for more details.
 * 
 *         You should have received a copy of the GNU General Public License
 *         along with Spiele; if not, write to the Free Software Foundation,
 *         Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
package de.yourwebs.sudoku;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URL;
import java.util.Timer;
import java.util.TimerTask;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineEvent;
import javax.sound.sampled.LineListener;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;

/**
 * The class provides the well known sudoku gui with a slider, that controlls
 * the difficulty of the game.
 * 
 * @author J&ouml;rg Czeschla, <a href="https://javabeginners.de" target=
 *         "_blank">javabeginners.de</a>
 *
 */
public class SudokuGUI extends JFrame implements KeyListener, LineListener {

    /*---------------------- Color Constants -------------------*/

    private final static Color FG_COLOR = new Color(30, 30, 30);
    private final static Color FG_COLOR_LIGHT_ACCESS = new Color(250, 250, 250);
    private final static Color FG_COLOR_DARK_ACCESS = new Color(30, 30, 30);
    private final static Color BG_COLOR_LIGHT = Color.WHITE;
    private final static Color BG_COLOR_LIGHT_ACCESS = Color.WHITE;
    private final static Color BG_COLOR_DARK = new Color(220, 220, 220);
    private final static Color BG_COLOR_DARK_ACCESS = Color.BLACK;
    private final static Color BORDER_COLOR = new Color(110, 110, 110);
    private final static Color BORDER_COLOR_LIGHT_ACCESS = new Color(250, 250,
            250);
    private final static Color BORDER_COLOR_DARK_ACCESS = new Color(30, 30, 30);
    private final static Color MISTAKE_BG_COLOR = new Color(255, 230, 210);
    private final static Color MISTAKE_BG_COLOR_ACCESS = new Color(255, 190,
            180);
    private final static Color MISTAKE_FG_COLOR = new Color(30, 30, 30);
    private final static Color MISTAKE_FG_COLOR_ACCESS = Color.BLACK;
    private final static Color HIGHLIGHT_BG_COLOR = new Color(100, 205, 100);

    /**
     * Background color of the light squares.<br>
     * See also {@link #switchColors(boolean)}
     */
    Color lightBgColor = BG_COLOR_LIGHT;

    /**
     * Background color of the dark squares.<br>
     * See also {@link #switchColors(boolean)}
     */
    Color darkBgColor = BG_COLOR_DARK;

    /**
     * Foreground color of the light squares. In non-accessible mode there is no
     * difference between light and dark squares.<br>
     * See also {@link #switchColors(boolean)}
     */
    Color lightFgColor = FG_COLOR;

    /**
     * Foreground color of the dark squares. In non-accessible mode there is no
     * difference between light and dark squares.<br>
     * See also {@link #switchColors(boolean)}
     */
    Color darkFgColor = FG_COLOR;

    /**
     * Border color of the dark squares in accessible mode. In non-accessible
     * mode the default color ist displayed.<br>
     * See also {@link #switchColors(boolean)}
     */
    Color borderColorLight = BORDER_COLOR;

    /**
     * Border color of the light squares in accessible mode. In non-accessible
     * mode the default color ist displayed.<br>
     * See also {@link #switchColors(boolean)}
     */
    Color borderColorDark = BORDER_COLOR;

    /*---------------------------- Default settings ------------------------*/

    /**
     * Number of cells in a row or col of a square - min. 3
     */
    private final int quadRows = 3;

    /**
     * Change to scale the font and border size - greater number means smaller
     * font
     */
    private int scaler = 5;

    /**
     * Default number of empty cells at startup. Set smaller number to make the
     * game more easy.
     */
    int difficulty = SudokuConstants.DIFFICULTY;

    /**
     * Milliseconds of highlighting a mistake before resetting the display to
     * the status before entering the wrong number.
     */
    int mistakeDelay = SudokuConstants.MISTAKE_DELAY;

    /**
     * Default font weight of the numbers
     */
    private int fontWeight = SudokuConstants.FONT_WEIGHT;

    /**
     * Show accessibility mode. Will be changed by the accessibility settings.
     */
    private boolean contrast = SudokuConstants.ACCESSIBILITY;

    /**
     * Give a hint if a false number is inserted.
     */
    private boolean mistakeHint = SudokuConstants.MISTAKE_HINT;

    /**
     * Play a sound when finishing the game successfully
     */
    private boolean isSuccessSoundPlayable = SudokuConstants.PLAY_SUCCESS_SOUND;

    /**
     * Voice output
     */
    private boolean isSpeechPlayable = SudokuConstants.PLAY_VOICE_OUTPUT;

    /**
     * Number of rows and cols in the entire game
     */
    private final int rowElements = quadRows * quadRows;
    
    /**
     * Count the number of mistakes to display it on the status bar
     */
    private int mistakes = 0;

    /**
     * Used to scale font and gui by the application's options dialog's slider.
     * Initialized by the default size of the frame set in the constructor.
     */
    private int faktor;

    /**
     * All cells ordered from left to right
     */
    private Element[][] elements;

    /**
     * The game's status bar
     */
    private StatusBar status;

    /**
     * The game's menu bar
     */
    private JMenuBar menubar;

    /**
     * the game's icon
     */
    private ImageIcon icon;

    /**
     * the business logic
     */
    private SudokuEngine engine;

    /**
     * Storing the configuration settings
     */
    private PropertiesManager pm;

    private PrintablePanel mainPanel;

    public SudokuGUI() {

        URL url = getClass().getClassLoader()
                .getResource(Config.ICON_PATH.value);
        icon = new ImageIcon(url);
        this.setIconImage(icon.getImage());

        loadProperties();

        engine = new SudokuEngine();
        engine.initEngine(quadRows);
        engine.createSudoku(0, 0);
        engine.preallocate(difficulty);

        initGUI();
        makeConfigFile();
        status.setEmptyCells(getEmptyCells());

        this.addComponentListener(new ComponentAdapter() {
            @Override
            public void componentResized(ComponentEvent e) {
                faktor = ((JFrame) e.getSource()).getSize().width / quadRows;
                setFaktor(faktor);
            }
        });
        this.setVisible(true);
    } // Konstruktor

    /**
     * Updates the game generating a new set of numbers and setting them to the
     * gui.
     * 
     * @param arr
     *            <code>byte[][]</code> the number array used to initialize the
     *            game.
     */
    void updateSudoku(byte[][] arr) {
        if (engine == null)
            return;
        if (arr.equals(engine.getResultArray())) {
            engine.setStartArray(engine.getResultArray());
            setElementsBGColor();
        }
        String s;
        for (int i = 0; i < rowElements; i++) {
            for (int j = 0; j < rowElements; j++) {
                s = (new Integer(arr[i][j]).toString());
                s = s.equals("-1") ? "" : s;
                elements[i][j].setText(s);
            }
        }
        // show the number of empty cells in the status bar.
        status.setEmptyCells(getEmptyCells());
    }

    /**
     * Initializes the gui
     */
    private void initGUI() {
        elements = new Element[rowElements][rowElements];
        mainPanel = new PrintablePanel(
                new GridLayout(rowElements, rowElements, 0, 0));
        status = new StatusBar(this, difficulty);
        status.setMistakeCount(0);
        initMainPanel();
        switchColors(contrast);
        setElementsBGColor();
        this.setLayout(new BorderLayout());
        this.add(mainPanel, BorderLayout.CENTER);
        this.add(status, BorderLayout.SOUTH);
        initMenubar();
        this.setTitle(Config.APP_NAME.value);
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setComponentSize();
    }

    /**
     * Creates an array of elements that represent the the games number fields.
     * The contents of the fields are fetched from an int-array, where -1 means
     * a blank space. Finally the main panel is filled by the elements.
     */
    private void initMainPanel() {
        if (engine == null)
            return;
        String s = "";
        byte intArr[][] = engine.getStartArray();
        if (intArr == null)
            return;
        for (int i = 0; i < rowElements; i++) {
            for (int j = 0; j < rowElements; j++) {
                elements[i][j] = new Element(this, faktor / scaler, fontWeight);
                elements[i][j].addKeyListener(this);
                String fw = pm.getProperty("fontWeight");
                if (fw != null && !fw.equals(""))
                    elements[i][j].setFontWeight(new Integer(fw).intValue());

                // setzt die Quadratnummer
                int quad = (i / quadRows) * quadRows + j / quadRows;
                elements[i][j].setQuad(quad);
                elements[i][j].setRow(i);
                elements[i][j].setCol(j);
                s = (new Integer(intArr[i][j]).toString());
                s = s.equals("-1") ? "" : s;
                elements[i][j].setText(s);
                mainPanel.add(elements[i][j]);
            }
        }
    }

    /**
     * Centers the component and sets its size to {@link #faktor}.
     */
    private void setComponentSize() {
        this.setSize(quadRows * faktor, quadRows * faktor);
        this.setLocationRelativeTo(null);
    }

    /**
     * Create a menubar
     */
    private void initMenubar() {
        menubar = new SudokuMenu(this);
        this.setJMenuBar(menubar);
    }

    /**
     * L&auml;dt die eingestellten Eigenschaften des Sudokus aus der
     * Optionen-Datei und castet sie in den notwendigen Datentyp.
     */
    private void loadProperties() {
        pm = new PropertiesManager();
        pm.loadProperties();
        if (pm.getProperty("sizeSlider") != null
                && !pm.getProperty("sizeSlider").equals(""))
            faktor = new Integer(pm.getProperty("sizeSlider"));
        if (pm.getProperty("difficulty") != null
                && !pm.getProperty("difficulty").equals(""))
            difficulty = new Integer(pm.getProperty("difficulty"));
        if (pm.getProperty("fontWeight") != null
                && !pm.getProperty("fontWeight").equals("")) {
            fontWeight = new Integer(pm.getProperty("fontWeight")).intValue();
        }
        if (pm.getProperty("contrast") != null
                && !pm.getProperty("contrast").equals("")) {
            contrast = new Boolean(pm.getProperty("contrast")).booleanValue();
        }
        if (pm.getProperty("mistakeHint") != null
                && !pm.getProperty("mistakeHint").equals("")) {
            mistakeHint = new Boolean(pm.getProperty("mistakeHint"))
                    .booleanValue();
        }
        if (pm.getProperty("isSuccessSoundPlayable") != null
                && !pm.getProperty("isSuccessSoundPlayable").equals("")) {
            isSuccessSoundPlayable = new Boolean(
                    pm.getProperty("isSuccessSoundPlayable")).booleanValue();
        }
        if (pm.getProperty("isSpeechPlayable") != null
                && !pm.getProperty("isSpeechPlayable").equals("")) {
            isSpeechPlayable = new Boolean(pm.getProperty("isSpeechPlayable"))
                    .booleanValue();
        }
        if (pm.getProperty("mistakeDelay") != null
                && !pm.getProperty("mistakeDelay").equals("")) {
            mistakeDelay = new Integer(pm.getProperty("mistakeDelay"))
                    .intValue();
        }
    }

    /**
     * Counts the number of empty cells.
     * 
     * @return <code>int</code> the number of empty cells
     */
    private int getEmptyCells() {
        int counter = 0;
        for (int i = 0; i < elements.length; i++) {
            for (int j = 0; j < elements[i].length; j++) {
                if (elements[i][j].getText().equals(""))
                    counter++;
            }
        }
        return rowElements * rowElements - counter;
    }

    /**
     * Ermittelt alle eingegebenen Zahlen, die <code>number</code> entsprechen
     * und markiert den Hintergrund von deren Zellen nachdem zun&auml;chst alle
     * Hintergr&uuml;nde auf die Default-Farbe zur&uuml;ckgesetzt wurden.
     * 
     * @param number
     *            <code>String</code>, der soeben eingegebene Wert der aktuellen
     *            Zelle, wie er durch den KeyListener zur&uuml;ck gegeben wurde.
     */
    public void highlightNumber(String number) {
        // reset background-color
        setElementsBGColor();
        for (int i = 0; i < elements.length; i++) {
            for (int j = 0; j < elements[i].length; j++) {
                if (!number.equals("")
                        && elements[i][j].getText().equals(number))
                    elements[i][j].setBackground(HIGHLIGHT_BG_COLOR);
                elements[i][j]
                        .setSelectionColor(elements[i][j].getBackground());
            }
        }
    }

    /**
     * L&auml;dt eine gespeicherte Belegung. Diese wird - je nach Belegung der
     * Variablen {@link Config#CONFIG_DIR} - im Heimatverzeichnis des Nutzers in
     * der Datei <code>.sudoku</code> abgelegt.
     */
    void open() {
        if (makeConfigFile()) {

            BufferedReader br;
            File f = new File(Config.SAVE_FILE.value);
            if (f.length() > 0) {
                try {
                    br = new BufferedReader(new FileReader(f));
                    for (int i = 0; i < elements.length; i++) {
                        for (int j = 0; j < elements[i].length; j++) {
                            elements[i][j].setText(br.readLine());
                        }
                    }
                    br.close();
                } catch (FileNotFoundException e1) {
                    e1.printStackTrace();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            } else {
                JOptionPane.showMessageDialog(this,
                        "Es ist noch kein Sudoku gespeichert.\n"
                                + "Laden Sie erst nach dem Abspeichern eines Spiels.",
                        "Laden nicht m\u00F6glich!", JOptionPane.OK_OPTION,
                        icon);
            }
        } else {
            System.err.println("Konfigurationsdatei kann nicht angelegt werden!"
                    + "\n" + "Laden der Einstellungen nicht m\u00F6glich!");
        }
    }

    /**
     * Speichert die aktuelle Belegung. Diese wird im Heimatverzeichnis des
     * Nutzers in der Datei <code>.sudoku</code> abgelegt.
     */
    void save() {
        PrintWriter pWriter = null;
        try {
            pWriter = new PrintWriter(new FileWriter(Config.SAVE_FILE.value));
            for (int i = 0; i < elements.length; i++) {
                for (int j = 0; j < elements[i].length; j++) {
                    pWriter.println(elements[i][j].getText());
                }
            }
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
            if (pWriter != null) {
                pWriter.flush();
                pWriter.close();
            }
        }
    }

    /**
     * Erzeugt eine Speicherdatei falls diese nicht vorhanden ist. In ihr kann
     * der aktuelle Belegungsstand gesichert werden.
     * 
     * @return <code>true</code> wenn die Datei vorhanden ist oder erfolgreich
     *         angelegt wurde.
     */
    private boolean makeConfigFile() {
        try {
            File dir = new File(Config.CONFIG_DIR.value);
            if (!dir.isDirectory() && !dir.mkdir()) {
                return false;

            }
            File cf = new File(Config.SAVE_FILE.value);

            if (cf.isFile() && cf.canRead() && cf.canWrite()) {
                return true;
            }
            if (cf.createNewFile()) {
                return true;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * Druckt das Sudoku aus
     */
    public void print() {
        PrinterJob printJob = PrinterJob.getPrinterJob();
        printJob.setPrintable(mainPanel);
        if (printJob.printDialog())
            try {
                printJob.print();
            } catch (PrinterException pe) {
                System.out.println("Error printing: " + pe);
            }
    }

    /**
     * Markiert falsche Zahlenbelegungen durch farbige Kennzeichnung der
     * horizontalen, vertikalen oder Teilquadrat-Elemente in denen bereits mit
     * der jeweiligen Zahl belegte Felder vorkommen. Der Wert der falschen Zahl
     * wird &uuml;bergeben und im doppelten teilelement mit
     * <code>mistakeFgColor</code> gekennzeichnet.
     * 
     * @param row
     *            <code>int</code>, Nummer der Reihe bei '0' beginnend
     * @param col
     *            <code>int</code>, Nummer der Spalte bei '0' beginnend
     * @param quad
     *            <code>int</code>, Nummer des Quadrats bei '0' beginnend
     * @param mistake
     *            <code>int</code>, numerisch kodierter Wert des Fehlerortes
     *            Dabei bedeuten:
     *            <dl>
     *            <dt>0</dt>
     *            <dd>horizontal match</dd>
     *            <dt>1</dt>
     *            <dd>vertical match</dd>
     *            <dt>2</dt>
     *            <dd>horizontal and vertical match</dd>
     *            <dt>3</dt>
     *            <dd>match in a square</dd>
     *            <dt>4</dt>
     *            <dd>horizontal match and match in a square</dd>
     *            <dt>5</dt>
     *            <dd>vertical match and match in a square</dd>
     *            <dt>6</dt>
     *            <dd>horizontal and vertical match and match in a square</dd>
     *            <dt>-1</dt>
     *            <dd>default</dd>
     *            </dl>
     * @param value
     *            <code>int</code>, Wert der falschen Zahl
     * 
     *            See also {@link SudokuEngine#check(int, int, int)}
     */
    private void highlightMistake(int row, int col, int quad, int mistake,
            int value) {
        // Vordergrundfarbe des falschen Zeichens einstellen
        Color fgColor = MISTAKE_FG_COLOR;
        Color bgColor = MISTAKE_BG_COLOR;
        if (contrast) {
            fgColor = MISTAKE_FG_COLOR_ACCESS;
            bgColor = MISTAKE_BG_COLOR_ACCESS;
        }
        // Iterate over all elements and mark multiple occurences
        for (int i = 0; i < elements.length; i++) {
            for (int j = 0; j < elements[i].length; j++) {
                switch (mistake) {
                case 0:
                    if (elements[i][j].getRow() == row) {
                        elements[i][j].setBackground(bgColor);
                        if (elements[i][j].getText().trim()
                                .equals(new Integer(value).toString())) {
                            elements[i][j].setForeground(fgColor);
                        }
                    }
                    break;
                case 1:
                    if (elements[i][j].getCol() == col) {
                        elements[i][j].setBackground(bgColor);
                        if (elements[i][j].getText().trim()
                                .equals(new Integer(value).toString())) {
                            elements[i][j].setForeground(fgColor);
                        }
                    }
                    break;
                case 2:
                    if (elements[i][j].getRow() == row
                            || elements[i][j].getCol() == col) {
                        elements[i][j].setBackground(bgColor);
                        if (elements[i][j].getText().trim()
                                .equals(new Integer(value).toString())) {
                            elements[i][j].setForeground(fgColor);
                        }
                    }
                    break;
                case 3:
                    if (elements[i][j].getQuad() == quad) {
                        elements[i][j].setBackground(bgColor);
                        if (elements[i][j].getText().trim()
                                .equals(new Integer(value).toString())) {
                            elements[i][j].setForeground(fgColor);
                        }
                    }
                    break;
                case 4:
                    if (elements[i][j].getRow() == row
                            || elements[i][j].getQuad() == quad) {
                        elements[i][j].setBackground(bgColor);
                        if (elements[i][j].getText().trim()
                                .equals(new Integer(value).toString())) {
                            elements[i][j].setForeground(fgColor);
                        }
                    }
                    break;
                case 5:
                    if (elements[i][j].getCol() == col
                            || elements[i][j].getQuad() == quad) {
                        elements[i][j].setBackground(bgColor);
                        if (elements[i][j].getText().trim()
                                .equals(new Integer(value).toString())) {
                            elements[i][j].setForeground(fgColor);
                        }
                    }
                    break;
                case 6:
                    if (elements[i][j].getCol() == col
                            || elements[i][j].getRow() == row
                            || elements[i][j].getQuad() == quad) {
                        elements[i][j].setBackground(bgColor);
                        if (elements[i][j].getText().trim()
                                .equals(new Integer(value).toString())) {
                            elements[i][j].setForeground(fgColor);
                        }
                    }
                    break;
                default:
                    return;
                }

                elements[i][j]
                        .setSelectionColor(elements[i][j].getBackground());
            }
        }
    }

    public int getFieldSize() {
        return rowElements;
    }

    public int getFaktor() {
        return faktor;
    }

    public int getFontWeight() {
        return fontWeight;
    }

    public boolean getContrast() {
        return contrast;
    }

    public boolean getMistakeHint() {
        return mistakeHint;
    }

    public boolean getSuccessSoundPlayable() {
        return isSuccessSoundPlayable;
    }

    public boolean getSpeechPlayable() {
        return isSpeechPlayable;
    }

    public String getMistakeDelay() {
        return new Integer(mistakeDelay).toString();
    }

    public SudokuEngine getEngine() {
        return engine;
    }

    public int getDifficulty() {
        return difficulty;
    }

    public StatusBar getStatus() {
        return status;
    }

    void setDifficulty(int difficulty) {
        this.difficulty = difficulty;
        pm.setProperty("difficulty", new Integer(difficulty).toString());
        pm.saveProperties();
    }

    void setFaktor(int faktor) {
        if (this.faktor != faktor) {
            this.faktor = faktor;
        }
        for (int i = 0; i < elements.length; i++) {
            for (int j = 0; j < elements[i].length; j++) {
                elements[i][j].setFaktor(faktor / scaler);
            }
        }
        pm.setProperty("sizeSlider", new Integer(faktor).toString());
        pm.saveProperties();
        setComponentSize();
    }

    /**
     * Steuert den Schriftschnitt des Sudokus
     * 
     * @param weight
     *            , boolean wenn true wird die Schrift fett dargestellt, sonst
     *            normal (plain)
     */
    void setFontWeight(boolean weight) {
        fontWeight = weight ? Font.BOLD : Font.PLAIN;
        for (int i = 0; i < elements.length; i++) {
            for (int j = 0; j < elements[i].length; j++) {
                elements[i][j].setFontWeight(fontWeight);
                elements[i][j].reloadFont();
            }
        }
        pm.setProperty("fontWeight", new Integer(fontWeight).toString());
        pm.saveProperties();
    }

    /**
     * Steuert die Dauer der Anzeige der Fehlerdarstellung
     * 
     * @param delay
     *            <code>String</code>, String-Repr&auml;sentanz des numerischen
     *            Anzeigewertes in Millisekunden.
     */
    void setMistakeDelay(String delay) {
        if (!delay.equals("") && delay.matches("[0-9]+")) {
            mistakeDelay = new Integer(delay).intValue();
            pm.setProperty("mistakeDelay", delay);
            pm.saveProperties();
        }
    }

    /**
     * Set the accessible mode with high contrast
     * 
     * @param selected
     *            <code>boolean</code>, checkbox status in options dialog
     */
    void setContrast(boolean selected) {
        switchColors(selected);
        pm.setProperty("contrast", new Boolean(selected).toString());
        pm.saveProperties();
        setElementsBGColor();
    }

    /**
     * Give a hint if a false value is inserted
     * 
     * @param selected
     *            <code>boolean</code>, checkbox status in options dialog
     */
    void setMistakeHint(boolean selected) {
        mistakeHint = selected;
        pm.setProperty("mistakeHint", new Boolean(selected).toString());
        pm.saveProperties();
    }

    void setSuccessSoundPlayable(boolean aku) {
        isSuccessSoundPlayable = aku;
        pm.setProperty("isSuccessSoundPlayable", new Boolean(aku).toString());
        pm.saveProperties();
    }

    void setSpeechPlayable(boolean speech) {
        isSpeechPlayable = speech;
        pm.setProperty("isSpeechPlayable", new Boolean(speech).toString());
        pm.saveProperties();
    }

    /**
     * Wechselt die Hintergrund- und Vordergrundfarbe
     * 
     * @param selected
     *            <code>boolean</code>, <code>true</code> setzt die Farben auf
     *            den Kontrast-Modus. Hier ist der Hintergrund schwarz und die
     *            Vordergrundfarbe weiss. Das Fehler-Highlighting wird in diesem
     *            Fall auf ein reines Rot gesetzt.
     */
    private void switchColors(boolean selected) {
        contrast = selected;
        lightFgColor = selected ? FG_COLOR_DARK_ACCESS : FG_COLOR;
        darkFgColor = selected ? FG_COLOR_LIGHT_ACCESS : FG_COLOR;
        lightBgColor = selected ? BG_COLOR_LIGHT_ACCESS : BG_COLOR_LIGHT;
        darkBgColor = selected ? BG_COLOR_DARK_ACCESS : BG_COLOR_DARK;
        borderColorLight = selected ? BORDER_COLOR_LIGHT_ACCESS : BORDER_COLOR;
        borderColorDark = selected ? BORDER_COLOR_DARK_ACCESS : BORDER_COLOR;
    }

    /**
     * Controlls the switching of the square's background color.
     * 
     * See also {@link #switchColors(boolean) switchColors}
     */
    public void setElementsBGColor() {
        if (elements == null)
            return;
        for (int i = 0; i < rowElements; i++) {
            for (int j = 0; j < rowElements; j++) {
                int quad = i / quadRows * quadRows + j / quadRows;

                if (quad % 2 == 0) {
                    elements[i][j].setForeground(lightFgColor);
                    elements[i][j].setBackground(lightBgColor);
                    elements[i][j].setBorderColor(borderColorDark);
                } else {
                    elements[i][j].setForeground(darkFgColor);
                    elements[i][j].setBackground(darkBgColor);
                    elements[i][j].setBorderColor(borderColorLight);
                }
            }
        }
    }

    /**
     * Aus dem Menu heraus das Sudoku neu belegen
     */
    void initNewSudoku() {
        if (engine == null)
            return;
        engine.initFields();
        engine.createSudoku(0, 0);
        engine.preallocate(getDifficulty());
        updateSudoku(engine.getStartArray());
        getStatus().setEmptyCells(getEmptyCells());
        getStatus().setMistakeCount(0);
    }

    /**
     * Pr&uuml;ft, ob das Sudoku korrekt gel&ouml;st ist. Dies ist der Fall,
     * wenn jede Zalhl einmalig in einer Zeile, einer Spalte und einem Teil-
     * quadrat vorhanden ist und kein Leerfeld mehr auftritt.
     * 
     * @return <code>false</code> if there is any field entry that does not
     *         match the value of the appropriate result array's index,
     *         otherwise <code>true</code>.
     */
    private boolean verifySuccess() {
        if (engine == null)
            return false;
        byte[][] result = engine.getResultArray();
        boolean handle = true;
        for (int i = 0; i < result.length; i++) {
            for (int j = 0; j < result[i].length; j++) {
                if (!elements[i][j].getText()
                        .equals(new Integer(result[i][j]).toString())) {
                    handle = false;
                }
            }
        }
        return handle;
    }

    /**
     * Soundausgabe des fertig gel&ouml;sten Sudokus
     * 
     * @param soundFile
     *            <code>String</code> the sound file
     */
    void playSound(String soundFile) {
        try {
            URL url = getClass().getClassLoader().getResource(soundFile);
            if (url == null)
                return;
            AudioInputStream audioInputStream = AudioSystem
                    .getAudioInputStream(url);
            AudioFormat af = audioInputStream.getFormat();
            Clip clip = null;
            DataLine.Info info = new DataLine.Info(Clip.class,
                    audioInputStream.getFormat(),
                    ((int) audioInputStream.getFrameLength()
                            * af.getFrameSize()));

            clip = (Clip) AudioSystem.getLine(info);
            clip.addLineListener(this);
            clip.open(audioInputStream);
            clip.start();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Activates a timer, that schedules the reset of the gui after showing the
     * mistake. The mistake is shown by changing the background-color of the
     * affected row/col/square.<br>
     * See also {@link SudokuGUI#highlightMistake(int, int, int, int, int)}
     * 
     * @param el
     *            <code>Element</code>, the Element-Object causing the mistake.
     * 
     */
    private void handleMistake(Element el) {
        if (engine == null)
            return;
        // Remove the wrong entry from the start array
        engine.getStartArray()[el.getRow()][el.getCol()] = 0;
        Timer t = new Timer();
        MistakeTimer mt = new MistakeTimer(this, el);
        t.schedule(mt, mistakeDelay);
        status.setMistakeCount(++mistakes);
    }

    /********************* Event-Handling ****************/

    @Override
    public void keyReleased(KeyEvent e) {
        if (engine == null)
            return;
        Element el = ((Element) e.getSource());
        int row = el.getRow();
        int col = el.getCol();

        // Navigation per Tastatur
        if (e.getKeyCode() == KeyEvent.VK_UP) {
            row = row > 0 ? row : 0;
            elements[row - 1][col].requestFocus();
            highlightNumber(elements[row - 1][col].getText());
            return;
        }
        if (e.getKeyCode() == KeyEvent.VK_DOWN) {
            elements[row + 1][col].requestFocus();
            highlightNumber(elements[row + 1][col].getText());
            return;
        }
        if (e.getKeyCode() == KeyEvent.VK_LEFT) {
            elements[row][col - 1].requestFocus();
            highlightNumber(elements[row][col - 1].getText());
            return;
        }
        if (e.getKeyCode() == KeyEvent.VK_RIGHT) {
            elements[row][col + 1].requestFocus();
            highlightNumber(elements[row][col + 1].getText());
            return;
        }

        // Texteingabe
        String s = el.getText().trim();

        // nur ein- oder mehrstellige Zahlen zulassen
        if (!s.matches("[1-9]") && !s.equals("")) {
            System.out.println("Nur Zahlen zwischen 1 und 9 sind erlaubt!");
            JOptionPane.showMessageDialog(this,
                    "Bitte nur Zahlen zwischen 1 und " + rowElements
                            + " eingeben!",
                    "Nur Zahlen sind erlaubt!", JOptionPane.OK_OPTION, icon);
            handleMistake(el);
            return;
        }

        // Anzahl der leeren Zellen anzeigen
        status.setEmptyCells(getEmptyCells());
        // Abbruch bei leerer Zelle
        if (s.equals("")) {
            return;
        }

        // Array mit Werten updaten und gleiche Zahlen highlighten
        Byte b = new Byte(s).byteValue();
        engine.setValueToStartArrayPos(row, col, b);
        highlightNumber(s);

        // Fehlerbehandlung, b enthaelt die zu pruefende Zahl
        if (mistakeHint) {
            int mistake = engine.check(row, col, b);
            highlightMistake(row, col, el.getQuad(), mistake, b);

            if (mistake > -1) {
                handleMistake(el);
            }
        }

        if (verifySuccess()) {
            if (isSuccessSoundPlayable)
                playSound(Config.SUCCESS_SOUND.value);
            new Success(this);
        } else {
            if (isSpeechPlayable)
                playSound(Config.SOUND_PATH.value + s + ".wav");
        }
    }

    @Override
    public void keyTyped(KeyEvent arg0) {
    }

    @Override
    public void keyPressed(KeyEvent e) {
        // Setzt die Hintergrundfarbe auf den Standard zurueck
        setElementsBGColor();
    }

    @Override
    public void update(LineEvent event) {
    }
}

class MistakeTimer extends TimerTask {
    // int counter = 0;
    SudokuGUI parent;
    Element el;

    public MistakeTimer(SudokuGUI parent, Element el) {
        this.parent = parent;
        this.el = el;
    }

    @Override
    public void run() {
        if (el.getText().equals(""))
            return;
        el.setText("");
        parent.setElementsBGColor();
    }
}
